#include <FastLED.h>
#include <LiquidCrystal_I2C.h>

#define LED_PIN  6           
#define NUM_LEDS 30
#define LCD_COL  15
#define BUTTON_PIN 2
#define INPUT_WINDOW 200

CRGB leds[NUM_LEDS];
int buttonStatus = 0;
long lastDebounceTime = 0;
long debounceTime = 0;
long debounceDelay = 300;
int mode = 0;
int state = 0;
unsigned long startMillis;
unsigned long currentMillis;
int displayLcdTimer = 0;
int displayLcdButton = 0;

LiquidCrystal_I2C lcd(0x27,16,2);

byte customChar[8] = {
	0b00000,
	0b01010,
	0b11111,
	0b11111,
	0b01110,
	0b00100,
	0b00000,
	0b00000
};

struct CRGB colors[] = {CRGB(255, 0 , 0), CRGB(255, 127 , 0), CRGB(255, 255, 0), CRGB(0, 255, 0),  CRGB(0, 0, 255),
CRGB(75, 0, 130),  CRGB(148, 0, 211), CRGB (224, 0, 255), CRGB (255, 0, 96)};


// https://www.arduinoslovakia.eu/application/timer-calculator
void setupTimerButton() {
  cli();
  TCCR1A = 0;
  TCCR1B = 0;
  TCNT1 = 0;

  // 1 Hz (16000000/((15624+1)*1024))
  OCR1A = 15624;
  // CTC
  TCCR1B |= (1 << WGM12);
  // Prescaler 1024
  TCCR1B |= (1 << CS12) | (1 << CS10);
  // Output Compare Match A Interrupt Enable
  TIMSK1 |= (1 << OCIE1A);
  sei();
}

void setup() {
  Serial.begin(9600);

  // setup led strip
  FastLED.addLeds<WS2812, LED_PIN, GRB>(leds, NUM_LEDS);
  for (int i = NUM_LEDS/2; i >= 0; i--) {
    leds[i] = colors[0];
    leds[NUM_LEDS-i] = colors[0];
    FastLED.show();
  }

  // setup button
  setupTimerButton();
  buttonStatus = 0;
  lastDebounceTime = millis();

  pinMode(BUTTON_PIN, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(BUTTON_PIN), myISR, FALLING);
  state = 0;
  mode = 0;
  startMillis = millis();

  // setup lcd
  lcd.init(); //initialize the lcd
  lcd.backlight(); //open the backlight
  lcd.createChar(0, customChar);
  lcd.setCursor(0,0);
  lcd.print("Mode: Down Up");
}

void myISR() {
  if (digitalRead(BUTTON_PIN) == LOW && (millis() - lastDebounceTime) > debounceDelay) {
    while (digitalRead(BUTTON_PIN) == LOW) {
      // wait for release
    }
    if (buttonStatus == 1) {
      buttonStatus = 0;
      lastDebounceTime = millis();
    } else {
      buttonStatus = 1;
      lastDebounceTime = millis();
    }

    // reset timer and move to the next mode
    startMillis = currentMillis; 
    state++;
    mode++;
    if (mode > 3) {
        state = 0;
        mode = 0;
    }
    // if button pressed change the text
    displayLcdButton = 1;
  }
}

// changes mode if in 10s no the button wasn't pushed
ISR(TIMER1_COMPA_vect) {
  currentMillis = millis();
  if (currentMillis - startMillis >= 10000) {
    displayLcdTimer = 1;
    state++;
    mode++;
    if (mode > 3) {
        state = 0;
        mode = 0;
    }
    startMillis = currentMillis;
  }
}

void displayLcdMode() {
  lcd.clear();
  lcd.setCursor(0,0);
  if (mode == 0) {
    lcd.print("Mode: Down Up");
  } else if (mode == 1) {
    lcd.print("Mode: Up Down");
  } else if (mode == 2) {
    lcd.print("Mode: Int Ext");
  } else if (mode == 3) {
    lcd.print("Mode: Ext Int");
  }
}

void displayHearts(int noHearts) {
  for (int i = 0; i < noHearts; i++) {
    lcd.setCursor(i,1);
    lcd.write((byte)0);
  }
}

void displayLedsAndHearts0(int noLeds, int counter) {
  for (int i = 0; i <= noLeds; i++)
    leds[i] = colors[counter];
  displayHearts(counter);
}

// 0
void downUp (int s) {
  int counter = 0;
  if ((s>=40)&&(s<=45))
    displayLedsAndHearts0(4, counter);

  counter++;
  if ((s>=45)&&(s<=50))
    displayLedsAndHearts0(7, counter);

  counter++;
  if ((s>=55)&&(s<=60))
    displayLedsAndHearts0(10, counter);

  counter++;
  if ((s>=65)&&(s<=70))
    displayLedsAndHearts0(13, counter);

  counter++;
  if((s>=70)&&(s<=72))
    displayLedsAndHearts0(17, counter);

  counter++;
  if((s>=73)&&(s<=74))
    displayLedsAndHearts0(20, counter);
  
  counter++;
  if((s>=75)&&(s<=77))
    displayLedsAndHearts0(23, counter);

  counter++;
  if((s>=78)&&(s<=79))
    displayLedsAndHearts0(27, counter);

  counter++;
  if((s>=80)&&(s<=82))
    displayLedsAndHearts0(30, counter);

  // clear leds
  leds[NUM_LEDS] = CRGB (0,0,0);
  for (int i = 0; i < NUM_LEDS; i++) 
    leds[i] = leds[i + 1];
  lcd.print(" ");
}

void displayLedsAndHearts1(int fristLed, int lastLed, int counter) {
  for (int i = NUM_LEDS - 1; i >= lastLed; i--) 
    leds[i] = colors[counter];
  displayHearts(counter);
  lcd.print(" ");
}

// 1
void upDown(int s) {
  int counter = 0; 
  if ((s>=40)&&(s<=45))
    displayLedsAndHearts1(NUM_LEDS - 1, 27, counter);

  counter++;
  if ((s>=45)&&(s<=50))
    displayLedsAndHearts1(26, 24, counter);

  counter++;
  if ((s>=55)&&(s<=60))
    displayLedsAndHearts1(23, 21, counter);

  counter++;
  if ((s>=65)&&(s<=70))
    displayLedsAndHearts1(20, 18, counter);

  counter++;
  if((s>=70)&&(s<=72))
    displayLedsAndHearts1(17, 16, counter);

  counter++;
  if((s>=73)&&(s<=74))
    displayLedsAndHearts1(13, 14, counter);

  counter++;
  if((s>=75)&&(s<=77))
    displayLedsAndHearts1(10, 10, counter);

  counter++;
  if((s>=78)&&(s<=79))
    displayLedsAndHearts1(7, 5, counter);

  counter++;
  if((s>=80)&&(s<=82))
    displayLedsAndHearts1(4, 0, counter);
  
  // clear leds
  leds[0] = CRGB (0,0,0);
  for (int i = 1; i <= NUM_LEDS - 1; i++) {
    leds[NUM_LEDS-i] = leds[NUM_LEDS-i-1];
  }
  lcd.print(" ");
}

void displayLedsAndHearts23(int led1, int led2, int counter, int lastLed) {
  leds[led1]= colors[counter];
  leds[led2]= colors[counter];
  for (int i = 0; i <= lastLed; i++) {
    leds[i] = leds[i+1];
    leds[NUM_LEDS-1-i] = leds[NUM_LEDS-i-2];
  }
  displayHearts(counter);
  lcd.print(" ");
}

// 2
void intExt (int s) {
  if ((s>=60)&&(s<=64)) {
    displayLedsAndHearts23(NUM_LEDS / 2, NUM_LEDS / 2 - 1, 1, 2);
  }
  else if ((s>=65)&&(s<=70)) {
    displayLedsAndHearts23(NUM_LEDS / 2, NUM_LEDS / 2 - 1, 2, 4);
  }
  else if((s>=70)&&(s<=72)) {
    displayLedsAndHearts23(NUM_LEDS / 2, NUM_LEDS / 2 - 1, 3, 6);
   }
  else if((s>=73)&&(s<=74)) {
    displayLedsAndHearts23(NUM_LEDS / 2, NUM_LEDS / 2 - 1, 4, 8);
  }
  else if((s>=75)&&(s<=77)) {
    displayLedsAndHearts23(NUM_LEDS / 2, NUM_LEDS / 2 - 1, 5, 10);
  }
  else if((s>=78)&&(s<=79)) {
    displayLedsAndHearts23(NUM_LEDS / 2, NUM_LEDS / 2 - 1, 6, 12);
  }
  else if((s>=80)&&(s<=82)) {
    displayLedsAndHearts23(NUM_LEDS / 2, NUM_LEDS / 2 - 1, 7, 13);
  }
  else {
    leds[NUM_LEDS / 2 - 1] = CRGB (0,0,0);
    leds[NUM_LEDS / 2] = CRGB ( 0,0,0);
  }

  for (int i = 0; i <= ((NUM_LEDS/2)-2); i++) {
    leds[i] = leds[i+1];
    leds[NUM_LEDS-1-i] = leds[NUM_LEDS-i-2];
  }
  lcd.print(" ");
}

void displayLedsAndHearts3(int led1, int led2, int counter, int firstLed) {
  leds[led1]= colors[counter];
  leds[led2]= colors[counter];
  for (int i = firstLed; i >= 0 ; i--) {
    leds[i + 1] = leds[i];
    leds[NUM_LEDS-i-2] = leds[NUM_LEDS-1-i];
  }
  displayHearts(counter);
  lcd.print(" ");
}

// 3
void extInt (int s) {
  if ((s>=62)&&(s<=64))
    displayLedsAndHearts3(0, NUM_LEDS - 1, 1, 2);
  else if ((s>=65)&&(s<=70))
    displayLedsAndHearts3(0, NUM_LEDS - 1, 2, 4);
   if((s>=70)&&(s<=72))
    displayLedsAndHearts3(0, NUM_LEDS - 1, 3, 6);
  else if((s>=73)&&(s<=74))
    displayLedsAndHearts3(0, NUM_LEDS - 1, 4, 8);
  else if((s>=75)&&(s<=77))
    displayLedsAndHearts3(0, NUM_LEDS - 1, 5, 10);
  else if((s>=78)&&(s<=79))
    displayLedsAndHearts3(0, NUM_LEDS - 1, 6, 12);
  else if((s>=80)&&(s<=82))
    displayLedsAndHearts3(0, NUM_LEDS - 1, 7, 13);
  else {
    leds[NUM_LEDS - 1] = CRGB (0,0,0);
    leds[0] = CRGB (0,0,0);
  }

  for (int i = NUM_LEDS / 2 - 2; i >= 0 ; i--) {
    leds[i + 1] = leds[i];
    leds[NUM_LEDS-i-2] = leds[NUM_LEDS-1-i];
  }
  lcd.print(" ");
}

void loop()
{
  unsigned int s = 0; // avgsound
  unsigned int inputMax = 0;
  unsigned int inputMin = 1024;

  //------------------------------------------------------------------------------------ Get sound sample
  // loop for the window
  for (unsigned int i = 0; i < INPUT_WINDOW; i++) {
    // read in a single value
    int value = analogRead(A0);

    // get the minimum and maximum value
    inputMin = min(inputMin, value);
    inputMax = max(inputMax, value);
    s += value;
  }

  s = s / INPUT_WINDOW;

//------------------------------------------------------------------------------------- display LCD

  if (displayLcdButton == 1) {
    displayLcdMode();
    displayLcdButton = 0;
  }
  if (displayLcdTimer == 1) {
    displayLcdMode();
    displayLcdTimer = 0;
  }
 Serial.println(s);
//------------------------------------------------------------------------------------- display leds
  if (s <= 30) {
    for (int i = 0; i < NUM_LEDS; i++) {
      leds[i] = CRGB (0, 0, 0);
      if (i < 15) {
        lcd.setCursor(i,1);
        lcd.print(" ");
      }
    }
  } else {
    if (mode == 0)
      downUp(s);
    else if (mode == 1)
      upDown(s);
    else if (mode == 2)
      intExt(s);
    else if (mode == 3)
      extInt(s);
  }
 FastLED.show();
}
